/*
 * foo_mcm570usbcontrol - Allow you to control Foobar 2000
 * from you Philips MC-M570 Hifi system
 * Copyright (C) 2006  Matthieu Achard
 * Developped from foo_controlserver by Jason Poloski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */


#include "mcm570usbcontrol.h"
#include "chandlecallbackrun.h"
#include "../foobar2000/SDK/foobar2000.h"
#include "../pfc/pfc.h"

// this will handle some events we are interested in
class play_callback_mcm570usbcontrol : public play_callback_static
{
public:
    virtual void FB2KAPI on_playback_starting(play_control::t_track_command p_command,bool p_paused) { 
		static_api_ptr_t<main_thread_callback_manager> cm;
	    cm->add_callback(new service_impl_t<CHandleCallbackRun>(CHandleCallbackRun::trackupdateonstart));
	};

    virtual void FB2KAPI on_playback_new_track(metadb_handle_ptr p_track)
    {
        static_api_ptr_t<main_thread_callback_manager> cm;

        cm->add_callback(new service_impl_t<CHandleCallbackRun>(CHandleCallbackRun::trackupdatefromptr, p_track));
    };

    virtual void FB2KAPI on_playback_stop(play_control::t_stop_reason p_reason)
    {
        // notify clients playback stopped if we stopped for real
        if (p_reason != playback_control::stop_reason_starting_another)
        {
            static_api_ptr_t<main_thread_callback_manager> cm;

            cm->add_callback(new service_impl_t<CHandleCallbackRun>(CHandleCallbackRun::trackupdateonstart));
        }
    };

    virtual void FB2KAPI on_playback_seek(double p_time)
    {
    };

    virtual void FB2KAPI on_playback_pause(bool p_state)
    {
        static_api_ptr_t<main_thread_callback_manager> cm;

        cm->add_callback(new service_impl_t<CHandleCallbackRun>(CHandleCallbackRun::trackupdateonstart));
    };

    // prolly dont want to handle this
    virtual void FB2KAPI on_playback_edited(metadb_handle_ptr p_track) { };

    // vbr change etc
    virtual void FB2KAPI on_playback_dynamic_info(const file_info & p_info) { };

    // stream file info changed perhaps
    // should probably handle this in the future
    virtual void FB2KAPI on_playback_dynamic_info_track(const file_info & p_info) { };

    // not handling
    virtual void FB2KAPI on_playback_time(double p_time) { };

    // handle volume changes
    virtual void FB2KAPI on_volume_change(float p_new_val)
    {
    };

    // events we are interested in
    unsigned get_flags()
    {
        return flag_on_playback_new_track | flag_on_playback_stop |
               flag_on_playback_pause;
    };

   FB2K_MAKE_SERVICE_INTERFACE(play_callback_mcm570usbcontrol, play_callback_static);
};

DECLARE_CLASS_GUID(play_callback_mcm570usbcontrol, 0xff3eb8d7, 0x97dd, 0x48b3, 0x80, 0xae, 0x15, 0x5b, 0x00, 0x99, 0x0c, 0x6d);

// used on intializtion of foobar to initalize plugin
// we create the server thread with this and kill it on close
class initquit_mcm570usbcontrol : public initquit
{
private:
    static HANDLE h_thread;
    static HANDLE endEvent;

    static DWORD WINAPI g_threadfunc(void* endEvent)
    {
        mcm570usbcontrol::start(static_cast<HANDLE>(endEvent));
        return 0;
    };

    inline static HANDLE create_thread(HANDLE event)
    {
        DWORD id;

        assert(core_api::is_main_thread());

        return h_thread = CreateThread(0, 0, g_threadfunc, event, 0, &id);
    };

    static void stop_thread()
    {
        assert(core_api::is_main_thread());

        if (h_thread)
        {
            WaitForSingleObject(h_thread, INFINITE);
            CloseHandle(h_thread);
            CloseHandle(endEvent);
            h_thread = 0;
            endEvent = 0;
        }
    };

public:
    static void do_init()
    {
        // create end event
        endEvent =  CreateEvent(NULL, TRUE, FALSE, NULL);

        // create our server! this will spawn additional threads
        // to handle connections to the server
        create_thread(endEvent);
    };
    static void do_quit()
    {
        // create event to stop the init
        SetEvent(endEvent);
        stop_thread();
    };
    virtual void on_init()
    {
        do_init();
    };

    virtual void on_quit()
    {
        do_quit();
    };

    FB2K_MAKE_SERVICE_INTERFACE(initquit_mcm570usbcontrol, initquit);
};

DECLARE_CLASS_GUID(initquit_mcm570usbcontrol,0xf888b7e9, 0xf120, 0x41de, 0x84, 0xd2, 0xeb, 0x40, 0xcb, 0xe3, 0xd5, 0xcb);

// set these to null initially
HANDLE initquit_mcm570usbcontrol::endEvent = NULL;
HANDLE initquit_mcm570usbcontrol::h_thread = NULL;


static initquit_factory_t<initquit_mcm570usbcontrol> foo;
//static preferences_page_factory_t<config_mcm570usbcontrol> foo2;
static play_callback_static_factory_t<play_callback_mcm570usbcontrol> foo3;

DECLARE_COMPONENT_VERSION("Philips MC-M570 USB Controler", "0.1",
                          "foo_mcm570usbcontrol - Allow you to control Foobar 2000\n"
						  "from you Philips MC-M570 Hifi system\n"
                          "Copyright (C) 2006  Matthieu Achard\n"
						  "Developped from foo_controlserver by Jason Poloski\n"
                          "\n"
                          "This program is free software; you can redistribute it and/or\n"
                          "modify it under the terms of the GNU General Public License\n"
                          "as published by the Free Software Foundation; either version 2\n"
                          "of the License, or (at your option) any later version.\n"
                          "\n"
                          "This program is distributed in the hope that it will be useful,\n"
                          "but WITHOUT ANY WARRANTY; without even the implied warranty of\n"
                          "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n"
                          "GNU General Public License for more details.\n"
                          "\n"
                          "You should have received a copy of the GNU General Public License\n"
                          "along with this program; if not, write to the Free Software\n"
                          "Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.");
