/*
 * foo_mcm570usbcontrol - Allow you to control Foobar 2000
 * from you Philips MC-M570 Hifi system
 * Copyright (C) 2006  Matthieu Achard
 * Developped from foo_controlserver by Jason Poloski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "mcm570usbcontrol.h"
#include "chandlecallbackrun.h"
#include <stdio.h>
#include <windows.h>
#include <process.h>
#include <commctrl.h>
#include <string.h>
#include <time.h> 
#include <tchar.h>

void _cdecl strcpy_wc(char *Dest, wchar_t *Src)
{
while (*Src)
{
*Dest++ = *Src++;
}
*Dest = 0;
}

pfc::string8 const mcm570usbcontrol::m_versionNumber = "0.1";

//handle de la dll du pilote de la chaine
typedef int (__stdcall *MYPROC)(int, int, char*); //pointeur vers la fonction de uacb.dll
HINSTANCE hinstLib = LoadLibrary(_TEXT("uacb.dll")); 
MYPROC const mcm570usbcontrol::ProcAdd = (MYPROC) GetProcAddress(hinstLib, "_UacaSetMem@12"); 


// constructor
mcm570usbcontrol::mcm570usbcontrol()
{

};

// destructor
mcm570usbcontrol::~mcm570usbcontrol()
{

};

bool
mcm570usbcontrol::sendData(char *data)
{
	console::info("Envoi de donnees");
	if(ProcAdd!=NULL){
		(ProcAdd) (0xa4f4, strlen(data), data);
		(ProcAdd) (0xa4f2, 1, "\x00");
		(ProcAdd) (0xb0a0, 1, "\xc0");
	}
    Sleep(50);
    return true;
}

void
mcm570usbcontrol::generateTrackString(metadb_handle_ptr handle, pfc::string8& str)
{
    pfc::string8 field;
    static_api_ptr_t<playback_control> pbc;
    unsigned int i = 0;

    if (handle == NULL)
    {
        return;
    }

	service_ptr_t<titleformat_object> script;
    pfc::string8 text;

    // mutex on metafields
    static_api_ptr_t<titleformat_compiler>()->compile_safe(script, "%artist% - %title%");

    pbc->playback_format_title_ex(handle, NULL, text, script, NULL, play_control::display_level_all);

    str << text;
}

// song updated, notify a single client
void
mcm570usbcontrol::handleTrackUpdateOnStart()
{
	//etat de la lecture ?
	static_api_ptr_t<playback_control> pbc;

    //  are we paused
    if (pbc->is_paused())
    {
        sendData("\x02\xa2\x04");
		sendData("\x02\xa3\x01");
    }
    else
    {
        // not paused are we playing?
        if (pbc->is_playing())
        {
            sendData("\x02\xa2\x01");
        }
        else
        {
            // not playing either we must be stopped
            sendData("\x02\x02\x00");
        }
    }

	console::print("Mise a jour du Track");

	static_api_ptr_t<playlist_manager> plm;
    metadb_handle_ptr handlept;
	t_size indexPlayList = 0;
    t_size indexToUse = 0;
	t_size m_currTrackIndex = 0;
	t_size m_currPlaylistIndex = 0;
	pfc::string8 sendStr;           // string we are going to send

    if (plm->get_playing_item_location(&indexPlayList, &indexToUse))
    {
        plm->playlist_get_item_handle(handlept, indexPlayList, indexToUse);

        // update global curr track
        m_currTrackIndex = indexToUse;
        m_currPlaylistIndex = indexPlayList;
    }
    else
    {
        // bit of a crude way so that we always play a stopped track on start up
        // just get the focus of the last item in playlist
        m_currPlaylistIndex = plm->get_active_playlist();

        // handle
        plm->playlist_get_focus_item_handle(handlept, m_currPlaylistIndex);

        // update global current track
        m_currTrackIndex = plm->playlist_get_focus_item(m_currPlaylistIndex);
    }
    
    if (m_currTrackIndex == -1)
    {
        return;
    }

    plm->playlist_get_item_handle(handlept, m_currPlaylistIndex, m_currTrackIndex);

    if (handlept == NULL)
    {
        return;
    }
    else
    {
        // fill in string with track information
        generateTrackString(handlept, sendStr);
    }

	char track[512];
	strcpy(track, sendStr);
	char len[1];
	memset(len, strlen(track)+2, 1);
    char data[1024];
	memset(data, 0, 1024);
	memcpy(data, len, 1);
	memcpy(data+1, "\x40\x82", 2);
	memcpy(data+3, track, strlen(track));
	sendData(data);
}

void
mcm570usbcontrol::handleTrackUpdateFromPtr(metadb_handle_ptr p_track)
{
    pfc::string8 sendStr;

    // fill in string with track information
    generateTrackString(p_track, sendStr);
    char track[512];
	strcpy(track, sendStr);
    char len[1];
	memset(len, strlen(track)+2, 1);
    char data[1024];
	memset(data, 0, 1024);
	memcpy(data, len, 1);
	memcpy(data+1, "\x40\x82", 2);
	memcpy(data+3, track, strlen(track));
	sendData("\x02\xa2\x01");
	sendData(data);
}

// handle next command
void
mcm570usbcontrol::handleNextCommand()
{
	static_api_ptr_t<playback_control> pbc;
    pbc->start(play_control::track_command_next);
	if(!pbc->is_playing()){
		handleTrackUpdateOnStart();
	}
}

// handle prev command
void
mcm570usbcontrol::handlePrevCommand()
{
	static_api_ptr_t<playback_control> pbc;
    pbc->start(play_control::track_command_prev);
	if(!pbc->is_playing()){
		handleTrackUpdateOnStart();
	}
}

// handle stop command
void
mcm570usbcontrol::handleStopCommand()
{
	static_api_ptr_t<playback_control> pbc;
	//sendData("\x02\x02\x00");
    pbc->stop();
}

// handle pause command
void
mcm570usbcontrol::handlePauseCommand()
{
	static_api_ptr_t<playback_control> pbc;
	pbc->pause(true);
}

// handle play command
void
mcm570usbcontrol::handlePlayCommand()
{
	static_api_ptr_t<playback_control> pbc;
	pbc->pause(false);
}


void
mcm570usbcontrol::handleSeekTrackCommand(int delta)
{
	static_api_ptr_t<playback_control> pbc;
    pbc->playback_seek_delta(delta*4); //-4 ou +4 secs
}


// start up the server and listen for connections/spawn client handling threads
void
mcm570usbcontrol::start(HANDLE endEvent)
{
	console::info("Lancement du controle de la chaine");
	//thread principal du module. c'est l que a attend les commandes de la chaine
    static_api_ptr_t<main_thread_callback_manager> cm;
    //cm->add_callback(new service_impl_t<CHandleCallbackRun>(CHandleCallbackRun::trackupdateonstart));
	
	//_spawnl(_P_NOWAIT, "PhilipsRemote.exe", "philipsremote", "-debug", NULL);
	//Sleep(3000);

	STARTUPINFO si;
	PROCESS_INFORMATION pi;
	LPTSTR cmdLine=_tcsdup(TEXT("PhilipsRemote.exe -debug"));
    
	ZeroMemory(&pi, sizeof(pi));
	ZeroMemory(&si, sizeof(si));
	si.dwFlags=STARTF_USESHOWWINDOW;
	si.wShowWindow=SW_HIDE;
	si.cb=sizeof(si);

	if( !CreateProcess( NULL,   // No module name (use command line)
        cmdLine,      // Command line
        NULL,           // Process handle not inheritable
        NULL,           // Thread handle not inheritable
        FALSE,          // Set handle inheritance to FALSE
        0,              // No creation flags
        NULL,           // Use parent's environment block
        NULL,           // Use parent's starting directory 
        &si,            // Pointer to STARTUPINFO structure
        &pi )           // Pointer to PROCESS_INFORMATION structure
    ) 
    {
		console::info("Erreur ds la creation du process");
        return;
    }
    Sleep(3000);
	console::info("Controle de la chaine lance");

    HWND hwnd=FindWindow(_TEXT("PhilipsRemote_48DD8B83-1595-4f23-BF8C-74F9B42A65DA"), NULL);
	if(hwnd==(HWND)NULL){
		console::info("ERREUR 1 : FindWindow a pas trouve philipsremote");
		TerminateProcess(pi.hProcess, 0);
		return;
	}
	HWND lv=GetWindow(GetWindow(hwnd, GW_CHILD), GW_HWNDNEXT);
	if(lv==(HWND)NULL){
		console::info("ERREUR 2 : La listview existe pas ?");
		TerminateProcess(pi.hProcess, 0);
		return;
	}

	LVITEM lvi, *_lvi;
	char item[512];
	char *_item;
	long itemid=-1;
	long itemidreturn;
	HANDLE process=pi.hProcess;

	_lvi=(LVITEM*)VirtualAllocEx(process, NULL, sizeof(LVITEM),
                              MEM_COMMIT, PAGE_READWRITE);
	_item=(char*)VirtualAllocEx(process, NULL, 512, MEM_COMMIT,
                             PAGE_READWRITE);

	lvi.cchTextMax=512;

	int nbPrev=0;
    
	while(1)
	{
		DWORD normRet;
        normRet = WaitForMultipleObjects(1, &endEvent, FALSE, 100);
        if (normRet == WAIT_FAILED)
        {
            console::warning("Erreur inconnue");
            break;
        }
        // we got an exit event from the main thread, lets get out of here
        if (normRet == WAIT_OBJECT_0)
        {
            break;
        }

		itemidreturn=(long)SendMessage(lv, LVM_GETNEXTITEM, itemid, 0);
		if(itemidreturn!=-1)
		{
			itemid=itemidreturn;
			lvi.iSubItem=1;
			lvi.pszText=(LPWSTR)_item;
			WriteProcessMemory(process, _lvi, &lvi, sizeof(LVITEM), NULL);
			SendMessage(lv, LVM_GETITEMTEXT, (WPARAM)itemid, (LPARAM)_lvi);
			ReadProcessMemory(process, _item, item, 512, NULL);
			LPTSTR gt=(LPTSTR)item;
			char itemText[MAX_PATH];
			strcpy_wc(itemText, gt);
			switch(itemText[strlen(itemText)-1])
			{
			case '1':
				//play
				console::info("Command Play");
				cm->add_callback(new service_impl_t<CHandleCallbackRun>(CHandleCallbackRun::play));
				break;
			case '3':
				//pause
				console::info("Command Pause");
				cm->add_callback(new service_impl_t<CHandleCallbackRun>(CHandleCallbackRun::pause));
				break;
			case '2':
				//stop
				console::info("Command Stop");
				cm->add_callback(new service_impl_t<CHandleCallbackRun>(CHandleCallbackRun::stop));
				break;
			case '4':
				//prev
				console::info("Command Prev");
				cm->add_callback(new service_impl_t<CHandleCallbackRun>(CHandleCallbackRun::prev));
				break;
			case '5':
				//next
				console::info("Command Next");
				cm->add_callback(new service_impl_t<CHandleCallbackRun>(CHandleCallbackRun::next));
				break;
			case '6':
				//seeknext
				console::info("Command SeekNext");
				cm->add_callback(new service_impl_t<CHandleCallbackRun>(CHandleCallbackRun::seek, 1));
				break;
			case '7':
				//seekprev
				console::info("Command SeekPrev");
				if(nbPrev==3)
				{
					cm->add_callback(new service_impl_t<CHandleCallbackRun>(CHandleCallbackRun::seek, -1));
					nbPrev=0;
				}
				else
				{
					nbPrev++;
					continue;
				}
				break;
			default:
				break;
			}
		}
		Sleep(100);
	}
	TerminateProcess(process, 0);
    return;
}
